"use strict";
exports.__esModule = true;
var experimental_use_styled_component_1 = require("../experimental-use-styled-component");
var range = require("../../range");
var test_utils_1 = require("./test-utils");
var buildEditResponse = test_utils_1.createMultipleEditBuildFunction(experimental_use_styled_component_1["default"]);
describe('javascript', function () {
    it('simple', function () {
        var source = "\n    import r from 'r-dom'\n    const Parent = () => {\n      return r.div(['Hello'])\n    }\n";
        var r = buildEditResponse(source, range.create(4, 16, 4, 16));
        expect(r).toEqual([
            {
                newText: "r(StyledDiv, ['Hello'])",
                range: { end: { column: 29, line: 4 }, start: { column: 13, line: 4 } },
                title: 'Extract "div" to styled component'
            },
            {
                newText: '\nconst StyledDiv = styled("div", {});\n',
                range: { end: { column: 25, line: 2 }, start: { column: 25, line: 2 } },
                title: 'Extract "div" to styled component'
            },
            {
                newText: '\nimport { styled } from "styletron-react";\n',
                range: { end: { column: 25, line: 2 }, start: { column: 25, line: 2 } },
                title: 'Extract "div" to styled component'
            },
        ]);
    });
    it('with existed styled import', function () {
        var source = "\n    import {styled} from 'styletron-react'\n    import r from 'r-dom'\n    const Parent = () => {\n      return r.div(['Hello'])\n    }\n";
        var r = buildEditResponse(source, range.create(5, 16, 5, 16));
        expect(r).toEqual([
            {
                newText: "r(StyledDiv, ['Hello'])",
                range: { end: { column: 29, line: 5 }, start: { column: 13, line: 5 } },
                title: 'Extract "div" to styled component'
            },
            {
                newText: '\n\nconst StyledDiv = styled("div", {});\n',
                range: { end: { column: 25, line: 3 }, start: { column: 25, line: 3 } },
                title: 'Extract "div" to styled component'
            },
        ]);
    });
    it('with different tag ("code")', function () {
        var source = "\n    import {styled} from 'styletron-react'\n    import r from 'r-dom'\n    const Parent = () => {\n      return r.code(['Hello'])\n    }\n";
        var r = buildEditResponse(source, range.create(5, 16, 5, 16));
        expect(r).toEqual([
            {
                newText: "r(StyledCode, ['Hello'])",
                range: { end: { column: 30, line: 5 }, start: { column: 13, line: 5 } },
                title: 'Extract "code" to styled component'
            },
            {
                newText: '\n\nconst StyledCode = styled("code", {});\n',
                range: { end: { column: 25, line: 3 }, start: { column: 25, line: 3 } },
                title: 'Extract "code" to styled component'
            },
        ]);
    });
    it.only('with different rdom import', function () {
        var source = "\n    import {styled} from 'styletron-react'\n    import rd from 'r-dom'\n    const Parent = () => {\n      return rd.code(['Hello'])\n    }\n";
        var r = buildEditResponse(source, range.create(5, 16, 5, 16));
        expect(r).toEqual([
            {
                newText: "rd(StyledCode, ['Hello'])",
                range: { end: { column: 31, line: 5 }, start: { column: 13, line: 5 } },
                title: 'Extract "code" to styled component'
            },
            {
                newText: '\n\nconst StyledCode = styled("code", {});\n',
                range: { end: { column: 26, line: 3 }, start: { column: 26, line: 3 } },
                title: 'Extract "code" to styled component'
            },
        ]);
    });
});
