"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
exports.__esModule = true;
var Range = require("../range");
var Position = require("../pos");
var Ast = require("../ast");
var Patch = require("../patch");
var text_utils_1 = require("../text-utils");
function hasStyledImport(j, ast) {
    var styletronSpecifiers = getImportSpecifiers(j, ast, 'styletron-react');
    var styledSpecifier = styletronSpecifiers.find(function (s) {
        return j.ImportSpecifier.check(s) && j.Identifier.check(s.imported) && s.imported.name === 'styled';
    });
    return Boolean(styledSpecifier);
}
function getDefaultImportName(j, ast, module) {
    var node = Ast.findFirstNode(ast, j.ImportDeclaration, function (n) {
        return n.specifiers.length === 1 &&
            j.Identifier.check(n.specifiers[0].local) &&
            n.source.value === module;
    });
    var specifiers = getImportSpecifiers(j, ast, module);
    var defSpec = specifiers.find(function (s) { return j.ImportDefaultSpecifier.check(s); });
    if (!defSpec || !j.Identifier.check(defSpec.local)) {
        return null;
    }
    return defSpec.local.name;
}
function getImportSpecifiers(j, ast, module) {
    var node = Ast.findFirstNode(ast, j.ImportDeclaration, function (n) { return n.specifiers.length === 1 && n.source.value === module; });
    if (!node) {
        return [];
    }
    return node.specifiers;
}
function getTagName(j, node) {
    return j.MemberExpression.check(node.callee) &&
        j.Identifier.check(node.callee.object) &&
        j.Identifier.check(node.callee.property)
        ? node.callee.property.name
        : null;
}
var fixer = {
    suggestCodeAction: function (params) {
        var j = params.j, ast = params.ast;
        var rDomVar = getDefaultImportName(j, ast, 'r-dom');
        var node = Ast.findFirstNode(ast, j.CallExpression, function (n) {
            return n.loc !== null &&
                Range.isInside(params.selection, n.loc) &&
                j.MemberExpression.check(n.callee) &&
                j.Identifier.check(n.callee.object) &&
                n.callee.object.name === rDomVar &&
                j.Identifier.check(n.callee.property);
        });
        if (!node || !node.loc) {
            return null;
        }
        var tag = getTagName(j, node);
        return {
            title: "Extract \"" + tag + "\" to styled component",
            data: node.loc.start
        };
    },
    createEdit: function (params) {
        var data = params.data, ast = params.ast, j = params.j;
        var node = Ast.findFirstNode(ast, j.CallExpression, function (n) { return Ast.isOnPosition(n, data); });
        if (!node) {
            return null;
        }
        var rDomVar = getDefaultImportName(j, ast, 'r-dom');
        if (!rDomVar) {
            return null;
        }
        var tag = getTagName(j, node);
        if (!tag) {
            return null;
        }
        var componentName = 'Styled' + text_utils_1.capitalize(tag);
        var patches = [];
        var newNode = j.callExpression(j.identifier(rDomVar), __spread([
            j.identifier(componentName)
        ], node.arguments));
        patches.push(Patch.replaceNode(j, node, newNode));
        var lastImport = Ast.findLastNode(ast, j.ImportDeclaration, function () { return true; });
        var declarationStart = lastImport && lastImport.loc
            ? Position.create(lastImport.loc.end.line, lastImport.loc.end.column)
            : Position.create(1, 0);
        var styledComponentDeclaration = j.variableDeclaration('const', [
            j.variableDeclarator(j.identifier(componentName), j.callExpression(j.identifier('styled'), [j.literal(tag), j.objectExpression([])])),
        ]);
        var shouldInsertStyledImport = !hasStyledImport(j, ast);
        patches.push(Patch.insertLine(j, declarationStart, styledComponentDeclaration, {
            before: shouldInsertStyledImport ? 1 : 2
        }));
        if (shouldInsertStyledImport) {
            var styledImport = j.importDeclaration([j.importSpecifier(j.identifier('styled'))], j.literal('styletron-react'));
            patches.push(Patch.insertLine(j, declarationStart, styledImport));
        }
        return patches;
    }
};
exports["default"] = fixer;
